@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2020-2023 Julian Heinovski <heinovski@ccs-labs.org>
+# Copyright (c) 2020-2025 Julian Heinovski <heinovski@ccs-labs.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
@@ -21,12 +21,19 @@ import sys
 
 
 def main():
+    """
+    The main entry point of PlaFoSim's img2video script.
+
+    This script uses ffmpeg.
+    """
+
     if len(sys.argv) != 3:
-        print(f"usage: {sys.argv[0]} 'prefix_*.png' video.mp4")
+        print(f"A script for creating a video from screenshots with fmpeg.\n\nusage: {sys.argv[0]} 'prefix_*.png' video.mp4")
         sys.exit()
 
     print(f"Creating video {sys.argv[2]} from images with glob pattern {sys.argv[1]} ...")
 
+    # use ffmpeg to create a video from screenshot
     cmd = [
         'ffmpeg',
         '-framerate',
