@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2020-2023 Julian Heinovski <heinovski@ccs-labs.org>
+# Copyright (c) 2020-2025 Julian Heinovski <heinovski@ccs-labs.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
@@ -46,7 +46,7 @@ class FormationAlgorithm(ABC):
         from .infrastructure import Infrastructure
         from .platooning_vehicle import PlatooningVehicle  # noqa 811
 
-        assert isinstance(owner, PlatooningVehicle) or isinstance(owner, Infrastructure)
+        assert isinstance(owner, (Infrastructure, PlatooningVehicle))
         self._owner = owner  # the owning vehicle or infrastructure
 
     @property
@@ -58,14 +58,19 @@ class FormationAlgorithm(ABC):
         return self.__class__.__name__
 
     @abstractmethod
-    def add_parser_argument_group(cls, parser: argparse.ArgumentParser) -> argparse._ArgumentGroup:
+    def add_parser_argument_group(self, parser: argparse.ArgumentParser) -> argparse._ArgumentGroup:
         """
         Abstract method for performing any type of platoon formation (i.e., assignment calculation).
 
         This methods needs to be overridden in implementing sub-classes.
+
+        Returns
+        -------
+        argparse._ArgumentGroup
+            The specific argument group for this algorithm.
         """
 
-        sys.exit("ERROR: There shouldn't be an instance of this abstract base class!")
+        sys.exit(f"ERROR [{__name__}]: There shouldn't be an instance of this abstract base class!")
 
     @abstractmethod
     def do_formation(self):
@@ -75,7 +80,7 @@ class FormationAlgorithm(ABC):
         This methods needs to be overridden in implementing sub-classes.
         """
 
-        sys.exit("ERROR: There shouldn't be an instance of this abstract base class!")
+        sys.exit(f"ERROR [{__name__}]: There shouldn't be an instance of this abstract base class!")
 
     def finish(self):
         """
