@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2020-2023 Julian Heinovski <heinovski@ccs-labs.org>
+# Copyright (c) 2020-2025 Julian Heinovski <heinovski@ccs-labs.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
@@ -35,17 +35,17 @@ def check_and_prepare_gui():
     """
 
     if "SUMO_HOME" not in os.environ:
-        sys.exit("ERROR: Environment variable 'SUMO_HOME' is not declared! Have you installed SUMO?")
+        sys.exit(f"ERROR [{__name__}]: Environment variable 'SUMO_HOME' is not declared! Have you installed SUMO?")
     tools = os.path.join(os.environ["SUMO_HOME"], "tools")
     sys.path.append(tools)
 
     # check TraCI API version
-    import traci
+    import traci  # noqa C415
     api_version = traci.constants.TRACI_VERSION
     assert api_version
 
     if api_version != TRACI_SUPPORTED_VERSION:
-        sys.exit(f"ERROR: You are using an unsupported TraCI version ({api_version})! Make sure to install a SUMO version with TraCI API version {TRACI_SUPPORTED_VERSION}.")
+        sys.exit(f"ERROR [{__name__}]: You are using an unsupported TraCI version ({api_version})! Make sure to install a SUMO version with TraCI API version {TRACI_SUPPORTED_VERSION}.")
 
 
 def start_gui(config: str, step_length: float, play: bool = True):
@@ -56,6 +56,10 @@ def start_gui(config: str, step_length: float, play: bool = True):
     ----------
     config : str
         The name of the configuration file
+    step_length : float
+        The length of one simulation step in s
+    play : bool, optional
+        Whether to start the simulation automatically
     """
 
     binary = os.path.join(os.environ["SUMO_HOME"], "bin/sumo-gui")
@@ -73,7 +77,7 @@ def start_gui(config: str, step_length: float, play: bool = True):
         "--start",
         str(play),
     ]
-    import traci
+    import traci  # noqa C415
 
     traci.start(command)
 
@@ -107,8 +111,11 @@ def set_gui_window(road_length: int):
     road_length : int
         The length of the road in m
     """
-    import traci
 
+    import traci  # noqa C415
+
+    # set window bounday
+    # TODO make y-lim dependet on road length
     traci.gui.setBoundary(
         traci.gui.DEFAULT_VIEW,
         xmin=0,
@@ -116,8 +123,10 @@ def set_gui_window(road_length: int):
         xmax=road_length,
         ymax=25000,
     )
-    # TODO set zoom?
-    # traci.gui.setZoom(traci.gui.DEFAULT_VIEW, 1000)
+
+    # set zoom level based on raod length
+    zoom_factor = (1e8 / road_length) * 0.97
+    traci.gui.setZoom(traci.gui.DEFAULT_VIEW, zoom_factor)
 
 
 def add_gui_vehicle(
@@ -148,7 +157,7 @@ def add_gui_vehicle(
     """
 
     LOG.trace(f"Adding vehicle {vid} at {position},{lane} with {speed},{color}")
-    import traci
+    import traci  # noqa C415
 
     if vid not in traci.vehicle.getIDList():
         traci.vehicle.add(
@@ -184,7 +193,7 @@ def move_gui_vehicle(vid: int, position: float, lane: int, speed: float):
         The vehicle's new speed
     """
 
-    import traci
+    import traci  # noqa C415
 
     LOG.trace(f"Moving vehicle {vid} to {position},{lane} with {speed}")
     traci.vehicle.setSpeed(vehID=str(vid), speed=speed)
@@ -199,11 +208,11 @@ def gui_step(target_step: int, screenshot_filename: str = None):
     ----------
     target_step : int
         The target simulation step
-    screenshot_filename : str
+    screenshot_filename : str, optional
         The name of the screenshot file
     """
 
-    import traci
+    import traci  # noqa C415
 
     if screenshot_filename:
         file_name, file_extension = os.path.splitext(screenshot_filename)
@@ -229,7 +238,7 @@ def change_gui_vehicle_color(vid: int, color: tuple):
         The color (R, G, B) to use for the vehicle
     """
 
-    import traci
+    import traci  # noqa C415
 
     LOG.trace(f"Changing color of vehicle {vid} to {color}")
     traci.vehicle.setColor(str(vid), color)
@@ -245,7 +254,7 @@ def remove_gui_vehicle(vid: int):
         The id of the vehicle to remove
     """
 
-    import traci
+    import traci  # noqa C415
 
     LOG.trace(f"Removing vehicle {vid}")
     traci.vehicle.remove(str(vid), 2)
@@ -261,7 +270,7 @@ def prune_vehicles(keep_vids: list):
         The ids of the vehicle that should be kept
     """
 
-    import traci
+    import traci  # noqa C415
 
     for vid in set(map(int, traci.vehicle.getIDList())) - set(keep_vids):
         remove_gui_vehicle(vid)
@@ -272,7 +281,7 @@ def close_gui():
     Close the GUI.
     """
 
-    import traci
+    import traci  # noqa C415
 
     traci.close(False)
 
@@ -296,7 +305,7 @@ def draw_ramps(road_length: int, interval: int, labels: bool):
     width = 4
     height = 150
 
-    import traci
+    import traci  # noqa C415
 
     for x in range(0, road_length + 1, interval):
         traci.polygon.add(
@@ -336,7 +345,7 @@ def draw_road_end(road_length: int, label: bool):
     width = 4
     color = (255, 0, 0)
 
-    import traci
+    import traci  # noqa C415
 
     traci.polygon.add(
         "road-end",
@@ -375,7 +384,7 @@ def draw_infrastructures(infrastructures: list, labels: bool):
     width = 20
     color = (0, 0, 255)
 
-    import traci
+    import traci  # noqa C415
 
     for infrastructure in infrastructures:
         from .infrastructure import Infrastructure
