@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2020-2022 Julian Heinovski <heinovski@ccs-labs.org>
+# Copyright (c) 2020-2025 Julian Heinovski <heinovski@ccs-labs.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
@@ -23,7 +23,7 @@ from enum import Enum
 import numpy as np
 import pandas as pd
 
-from .util import assert_index_equal
+from plafosim.util import assert_index_equal
 
 # misc constants
 HIGHVAL = 1e15  # virtually infinite position or speed
@@ -41,7 +41,7 @@ class CF_Model(Enum):
     Car Following models that vehicles can use for their mobility.
     """
 
-    Human = 0  # safe speed
+    HUMAN = 0  # safe speed
     ACC = 1  # fixed time gap
     CACC = 2  # small fixed distance
 
@@ -50,35 +50,71 @@ class CF_Model(Enum):
 
 
 def is_gap_safe(
-    front_position,
-    front_speed,
-    front_max_deceleration,
-    front_length,
-    back_position,
-    back_speed,
-    back_max_acceleration,
-    step_length,
-):
+    front_position: float,
+    front_speed: float,
+    front_max_deceleration: float,
+    front_length: float,
+    back_position: float,
+    back_speed: float,
+    back_max_acceleration: float,
+    back_min_gap: float,
+    step_length: float,
+) -> bool:
     """
     Return whether the gap between the front and back vehicle is safe.
 
     Safe means:
     - the front vehicle can decelerate as hard as possible for one step
     - the back vehicle can accelerate as hard as possible for one step
-    - the will not crash
+    - the vehicle will not crash
 
-    Assumes euclidian/non-ballistic position updates.
+    Assumes euclidean/non-ballistic position updates.
+
+    Parameters
+    ----------
+    front_position : float
+        The position of the front vehicle in m
+    front_speed : float
+        The speed of the front vehicle in m/s
+    front_max_deceleration : float
+        The maximum deceleration of the front vehicle in m/s2
+    front_length : length
+        The length of the front vehicle in m
+    back_position : float
+        The position of the back vehicle in m
+    back_speed : float
+        The speed of the back vehicle in m/s
+    back_max_acceleration : float
+        The maximum acceleration of the back vehicle in m/s2
+    back_min_gap : float
+        The minimum gap of the back vehicle in m
+    step_length : float
+        The length of a simulation step in s
+
+    Returns
+    -------
+    bool : Whether the gap between the two vehicles is safe
     """
+
+    assert front_position >= 0
+    assert front_speed >= 0
+    assert front_max_deceleration >= 0
+    assert front_length > 0
+    assert back_position >= 0 or back_position == -HIGHVAL, f"{back_position}"
+    assert back_speed >= 0
+    assert back_max_acceleration >= 0
+    assert back_min_gap >= 0
+    assert step_length > 0
+
     next_front_position = (
         front_position
-        - front_length
         + (front_speed - front_max_deceleration * step_length) * step_length
     )
     next_back_position = (
         back_position
         + (back_speed + back_max_acceleration * step_length) * step_length
     )
-    return next_front_position > next_back_position
+    return (next_front_position - front_length) > next_back_position + back_min_gap
 
 
 # speed update components
@@ -86,15 +122,47 @@ def is_gap_safe(
 
 def safe_speed_df(vdf: pd.DataFrame) -> pd.Series:
     """
-    Compute the safe speed akkording to the Krauss model, DataFrame variant.
+    Compute the safe speed according to the Krauss model, DataFrame variant.
 
-    See S. Krauss, Microscopic Modeling of Traffic Flow: Investigation of Collision Free Vehicle Dynamics, 1998.
+    See S. Krauss, Microscopic Modeling of Traffic Flow: Investigation of Collision Free Vehicle Dynamics, 1998, Equation (4.21).
+    v_safe = v_leader + (gap - gap_desired) / tau_b + tau
+    gap_desired = tau * v_leader
+    tau_b = v_avg / b
+    b = typical deceleration of drivers
+    tau = reaction time of drivers
+
+    safety is guaranteed, if step length <= tau and gap_desired >= v_leader * step_length
+    <=> if the true reaction time (i.e., the length of one time step) is
+    smaller than or equal to the reaction time that each driver assumes
+
+    Parameters
+    ----------
+    vdf : pandas.DataFrame
+        The Dataframe containing the vehicles as rows
+        index: vid
+        columns: [position, length, lane, ..]
     """
 
-    tau_b = (vdf.predecessor_speed + vdf.speed) / (2 * vdf.max_deceleration)
+    if vdf.empty:
+        return pd.Series()
+
+    assert (vdf.predecessor_speed >= 0).all()
+    assert (vdf.speed >= 0).all()
+    assert (vdf.max_deceleration >= 0).all()
+    # HIGHVAL
+    assert (vdf[vdf.predecessor_vid == -1].predecessor_gap > 0).all()
+    assert (vdf.desired_gap > 0).all()
+    assert (vdf.desired_headway_time >= 0).all()
+
+    # average breaking time
+    b = vdf.max_deceleration  # TODO useful value? shouldn't this be the typical deceleration, e.g., max_deceleration / 2?
+    tau_b = ((vdf.predecessor_speed + vdf.speed) / 2) / b
+    # assumed reaction time
+    tau = vdf.desired_headway_time  # TODO useful value? shouldn't this be smaller than the headway time (e.g., 0.8s)?
+
     return vdf.predecessor_speed + (
         (vdf.predecessor_gap - vdf.desired_gap)
-        / (vdf.desired_headway_time + tau_b)
+        / (tau_b + tau)
     )
 
 
@@ -102,7 +170,14 @@ def speed_human_df(vdf: pd.DataFrame) -> pd.Series:
     """
     Compute new speed for human vehicles, DataFrame variant.
 
-    Basically just safe_speed, clamping is done outside this function.
+    Basically just safe speed, clamping is done outside this function.
+
+    Parameters
+    ----------
+    vdf : pandas.DataFrame
+        The Dataframe containing the vehicles as rows
+        index: vid
+        columns: [position, length, lane, ..]
     """
 
     # TODO: implement dawdling
@@ -110,32 +185,59 @@ def speed_human_df(vdf: pd.DataFrame) -> pd.Series:
     return speed_safe
 
 
-def speed_acc_df(vdf: pd.DataFrame, step_length: float = 1.0) -> pd.Series:
+def speed_acc_df(vdf: pd.DataFrame, step_length: float) -> pd.Series:
     """
     Compute new speed for ACC vehicles, DataFrame variant.
 
     Clamping is done outside this function.
 
     See Eq. 6.18 of R. Rajamani, Vehicle Dynamics and Control, 2nd. Springer, 2012.
+
+    Parameters
+    ----------
+    vdf : pandas.DataFrame
+        The Dataframe containing the vehicles as rows
+        index: vid
+        columns: [position, length, lane, ..]
+    step_length : float
+        The length of a simulation step in s
     """
 
+    if vdf.empty:
+        return pd.Series()
+
+    assert (vdf.predecessor_speed >= 0).all()
+    assert (vdf.speed >= 0).all()
+    assert (vdf.desired_gap > 0).all()
+    # HIGHVAL
+    assert (vdf[vdf.predecessor_vid == -1].predecessor_gap > 0).all()
+    assert (vdf.desired_headway_time >= 0).all()
+    assert step_length > 0
+
     acceleration = (
         vdf.predecessor_speed
         - vdf.speed
         - vdf.acc_lambda * (vdf.desired_gap - vdf.predecessor_gap)
-    ) / vdf.desired_headway_time
+    ) / vdf.desired_headway_time  # TODO make sure that this is the correct one for ACC
 
     return vdf.speed + acceleration * step_length
 
 
 def clamp_speed(
-    new_speed,
-    vdf,
-    step_length,
-):
+    new_speed: pd.Series,
+    vdf: pd.DataFrame,
+    step_length: float,
+) -> pd.Series:
     """
     Clamp (two-way limit) a new speed value to vehicle's maximum.
     """
+
+    assert (vdf.max_speed > 0).all()
+    assert (vdf.speed >= 0).all()
+    assert (vdf.max_acceleration >= 0).all()
+    assert (vdf.max_deceleration >= 0).all()
+    assert step_length > 0
+
     new_speed = np.min(
         [
             new_speed,
@@ -152,11 +254,11 @@ def clamp_speed(
         ],
         axis=0,
     )
-    assert not any(new_speed < 0)
+    assert (new_speed >= 0).all()
     return pd.Series(new_speed, index=vdf.index)
 
 
-def lane_predecessors(vdf, max_lane):
+def lane_predecessors(vdf: pd.DataFrame, max_lane: int) -> pd.DataFrame:
     """
     Find the current (potential) predecessor for each lane and each vehicle.
 
@@ -166,7 +268,19 @@ def lane_predecessors(vdf, max_lane):
 
     Preconditions:
     - vdf.sorted_values(['position', 'lane'], ascending=False)
+
+    Parameters
+    ----------
+    vdf : pandas.DataFrame
+        The Dataframe containing the vehicles as rows
+        index: vid
+        columns: [position, length, lane, ..]
+    max_lane : int
+        The largest lane id
     """
+
+    assert max_lane >= 0
+
     vdf = vdf.reset_index()
     return pd.DataFrame(
         {
@@ -180,7 +294,7 @@ def lane_predecessors(vdf, max_lane):
     ).set_index(vdf.vid)
 
 
-def lane_successors(vdf, max_lane):
+def lane_successors(vdf: pd.DataFrame, max_lane: int) -> pd.DataFrame:
     """
     Find the current (potential) successors for each lane and each vehicle.
 
@@ -190,7 +304,19 @@ def lane_successors(vdf, max_lane):
 
     Preconditions:
     - vdf.sorted_values(['position', 'lane'], ascending=False)
+
+    Parameters
+    ----------
+    vdf : pandas.DataFrame
+        The Dataframe containing the vehicles as rows
+        index: vid
+        columns: [position, length, lane, ..]
+    max_lane : int
+        The largest lane id
     """
+
+    assert max_lane >= 0
+
     vdf = vdf.reset_index()
     return pd.DataFrame(
         {
@@ -204,10 +330,18 @@ def lane_successors(vdf, max_lane):
     ).set_index(vdf.vid)
 
 
-def get_successors(vdf, successor_map, target_lane) -> pd.DataFrame:
+def get_successors(
+    vdf: pd.DataFrame,
+    successor_map: pd.DataFrame,
+    target_lane: pd.Series,
+) -> pd.DataFrame:
     """
     Return DataFrame of successors to the vehicles on a target lane.
     """
+
+    assert_index_equal(vdf, target_lane)
+    assert (target_lane >= 0).all()
+
     # get id of (the one) successor on the target lane
     successor_ids = pd.Series(
         # pick one value per vehicle using numpy, which is fast and stable
@@ -225,10 +359,18 @@ def get_successors(vdf, successor_map, target_lane) -> pd.DataFrame:
     )
 
 
-def get_predecessors(vdf, predecessor_map, target_lane) -> pd.DataFrame:
+def get_predecessors(
+    vdf: pd.DataFrame,
+    predecessor_map: pd.DataFrame,
+    target_lane: pd.Series,
+) -> pd.DataFrame:
     """
     Return DataFrame of successors to the vehicles on a target lane.
     """
+
+    assert_index_equal(vdf, target_lane)
+    assert (target_lane >= 0).all()
+
     # get id of (the one) successor on the target lane
     predecessor_ids = pd.Series(
         # pick one value per vehicle using numpy, which is fast and stable
@@ -248,7 +390,7 @@ def get_predecessors(vdf, predecessor_map, target_lane) -> pd.DataFrame:
 
 def compute_new_speeds(
     vdf: pd.DataFrame,
-    step_length: float = 1.0,
+    step_length: float,
 ) -> pd.Series:
     """
     Compute the new speed for all vehicles in the simulation.
@@ -256,8 +398,25 @@ def compute_new_speeds(
     Can compute "potential" new speed for different target lanes.
     Assume vdf already contains predecessor and successor data.
     Just pass the right predecessor/successor data for different lanes.
+
+    Parameters
+    ----------
+    vdf : pandas.DataFrame
+        The Dataframe containing the vehicles as rows
+        index: vid
+        columns: [position, length, lane, ..]
+    step_length : float
+        The length of a simulation step
+
+    Returns
+    -------
+    pandas.Series
+        The series of new speeds
+        index: vid
     """
 
+    assert step_length > 0
+
     # ensure no changes to vdf propagate out of this function
     vdf = vdf.copy()
     # derive common data for all cf models
@@ -266,22 +425,24 @@ def compute_new_speeds(
         # TODO: move desired_gap out of this function to avoid re-computation
         [
             vdf["min_gap"],
-            # using own speed instead of predecessor speed (like krauss) here
+            # using own speed instead of predecessor speed (like Krauss) here
             vdf["desired_headway_time"] * vdf["speed"],
         ],
         axis=0,
     )
     vdf["predecessor_gap"] = vdf["predecessor_rear_position"] - vdf["position"]
+    # HIGHVAL
+    assert (vdf[vdf.predecessor_vid == -1].predecessor_gap > 0).all()
 
     # Note:
     # we assume vehicles already have their max acceleration/deceleration
     # set/updated for individual driving or platooning.
     # I.e., platoons share max acceleration/deceleration.
-    # We also assume that evary vehicle has speed and postion values of its
+    # We also assume that every vehicle has speed and position values of its
     # predecessor. If there is none, the values should just be large defaults.
-    # The CF functions shouls be able to deal with this inherently.
+    # The CF functions should be able to deal with this inherently.
 
-    m_human = vdf.cf_model == CF_Model.Human
+    m_human = vdf.cf_model == CF_Model.HUMAN
     m_acc = vdf.cf_model == CF_Model.ACC
     m_cacc = vdf.cf_model == CF_Model.CACC
 
@@ -291,8 +452,8 @@ def compute_new_speeds(
     # and should avoid re-sorting the result -- but maybe benchmark this later
     new_speed = vdf.speed.copy()
     new_speed.loc[m_human] = speed_human_df(vdf.loc[m_human])
-    new_speed.loc[m_acc] = speed_acc_df(vdf.loc[m_acc])
-    # TODO: apply clamping only to human and acc vehicles (not cacc)
+    new_speed.loc[m_acc] = speed_acc_df(vdf.loc[m_acc], step_length)
+    # TODO: apply clamping only to human and ACC vehicles (not CACC)
     new_speed.loc[m_cacc] = new_speed[vdf.loc[m_cacc].leader_id].values
 
     # clamp speed by common constraints
@@ -305,7 +466,11 @@ def compute_new_speeds(
     return pd.Series(new_speed, index=vdf.index)
 
 
-def compute_lane_changes(vdf, max_lane, step_length=1.0):
+def compute_lane_changes(
+    vdf: pd.DataFrame,
+    max_lane: int,
+    step_length: float,
+) -> pd.DataFrame:
     """
     Find desired and safe lane changes.
 
@@ -322,7 +487,29 @@ def compute_lane_changes(vdf, max_lane, step_length=1.0):
     if ((favorable(i->j) or (rand < p_change)) and safe(i->j)) then change(i->j)
     for vehicles on the right lane:
     if (v > v^0_safe) and (not congested) then v <- v^0_safe
+
+    Parameters
+    ----------
+    vdf : pandas.DataFrame
+        The Dataframe containing the vehicles as rows
+        index: vid
+        columns: [position, length, lane, ..]
+    max_lane : int
+        The largest lane id
+    step_length : float
+        The length of a simulation step
+
+    Returns
+    -------
+    pandas.DataFrame
+        The Dataframe containing the vehicles as rows
+        index: vid
+        columns: [lane, reason]
     """
+
+    assert max_lane >= 0
+    assert step_length > 0
+
     # TODO: check order of computation for performance
     # TODO: skip computations for CACC vehicles
 
@@ -331,7 +518,7 @@ def compute_lane_changes(vdf, max_lane, step_length=1.0):
 
     ## SPEED GAIN
 
-    # derive map of predecessrs and successors
+    # derive map of predecessors and successors
     predecessor_map = lane_predecessors(vdf_tmp, max_lane)
     successor_map = lane_successors(vdf_tmp, max_lane)
 
@@ -349,6 +536,7 @@ def compute_lane_changes(vdf, max_lane, step_length=1.0):
         predecessor_map=predecessor_map,
         target_lane=vdf_tmp.lane,
     ).rename(columns=lambda col: "predecessor_" + col)
+    assert predecessor_current[predecessor_current.predecessor_vid != -1].predecessor_vid.is_unique
 
     # predict speed for potential maneuvers
     speed_left = compute_new_speeds(
@@ -395,7 +583,7 @@ def compute_lane_changes(vdf, max_lane, step_length=1.0):
 
     ## KEEP RIGHT
 
-    # derive map of predecessrs and successors
+    # derive map of predecessors and successors
     predecessor_map = lane_predecessors(vdf_tmp, max_lane)
     successor_map = lane_successors(vdf_tmp, max_lane)
 
@@ -453,7 +641,7 @@ def compute_lane_changes(vdf, max_lane, step_length=1.0):
     vdf_tmp.loc[m_cacc, 'lane'] = vdf_tmp.loc[vdf_tmp.loc[m_cacc, 'leader_id'], 'lane'].values
     vdf_tmp.loc[m_cacc, 'reason'] = vdf_tmp.loc[vdf_tmp.loc[m_cacc, 'leader_id'], 'reason'].values
 
-    # vehicls are only allowed to perform one lane change
+    # vehicles are only allowed to perform one lane change
     assert not (speed_gain & keep_right).any()
 
     return vdf_tmp[['lane', 'reason']]
@@ -462,7 +650,7 @@ def compute_lane_changes(vdf, max_lane, step_length=1.0):
 # Stuff to remove
 #
 # Just temporary helpers for migration to fully vectorized code.
-# Remove once no longer needd
+# Remove once no longer needed.
 
 SAFE_SPEED_DF = namedtuple(
     "SAFE_SPEED_DF",
@@ -524,7 +712,7 @@ def safe_speed(
 
 def clip_position(position: pd.Series, vdf: pd.DataFrame) -> pd.Series:
     """
-    Return the clipped positions (i.e., by arrival position) of vehicles within a pandas DataFrame.
+    Return the clipped positions (i.e., by arrival position) of vehicles within a pandas Dataframe.
 
     Parameters
     ----------
@@ -533,9 +721,15 @@ def clip_position(position: pd.Series, vdf: pd.DataFrame) -> pd.Series:
         index: vid
         columns: [position, length, lane, ..]
     vdf : pandas.DataFrame
-        The dataframe containing the vehicles as rows
+        The Dataframe containing the vehicles as rows
         index: vid
         columns: [position, length, lane, ..]
+
+    Returns
+    -------
+    pandas.Series
+        The series of clipped positions
+        index: vid
     """
 
     assert_index_equal(position, vdf)
@@ -553,9 +747,9 @@ def clip_position(position: pd.Series, vdf: pd.DataFrame) -> pd.Series:
     return clipped_position
 
 
-def update_position(vdf: pd.DataFrame, step_length: int) -> pd.DataFrame:
+def update_position(vdf: pd.DataFrame, step_length: float) -> pd.DataFrame:
     """
-    Update the position of vehicles within a pandas DataFrame.
+    Update the position of vehicles within a pandas Dataframe.
 
     This is based on Krauss' single lane traffic:
     adjust position (move)
@@ -564,13 +758,22 @@ def update_position(vdf: pd.DataFrame, step_length: int) -> pd.DataFrame:
     Parameters
     ----------
     vdf : pandas.DataFrame
-        The dataframe containing the vehicles as rows
+        The Dataframe containing the vehicles as rows
         index: vid
         columns: [position, length, lane, ..]
-    step_length : int
+    step_length : float
         The length of the simulated step
+
+    Returns
+    -------
+    pandas.DataFrame
+        The Dataframe containing the vehicles as rows
+        index: vid
+        columns: [position, length, lane, ..]
     """
 
+    assert step_length > 0
+
     position = vdf.position + (vdf.speed * step_length)
     vdf["position"] = clip_position(position, vdf)
     return vdf
@@ -583,11 +786,18 @@ def get_crashed_vehicles(vdf: pd.DataFrame) -> list:
     Parameters
     ----------
     vdf : pandas.DataFrame
-        The dataframe containing the vehicles as rows
+        The Dataframe containing the vehicles as rows
         index: vid
         columns: [position, length, lane, ..]
+
+    Returns
+    -------
+    list(int) : the list of vehicles that crashed
     """
 
+    if vdf.empty:
+        return []
+
     # sort vehicles by their position
     vdf = vdf.sort_values("position", ascending=False)
 
@@ -601,7 +811,7 @@ def get_crashed_vehicles(vdf: pd.DataFrame) -> list:
         return []
     vdf = vdf[vdf.lane.isin(list(lanes_to_keep))]
 
-    # TODO should we just use the precessor / successor
+    # TODO should we just use the predecessor / successor
 
     # calculate vehicles with a crash in their back
     groupby = vdf.groupby("lane")
