@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2020-2023 Julian Heinovski <heinovski@ccs-labs.org>
+# Copyright (c) 2020-2025 Julian Heinovski <heinovski@ccs-labs.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
@@ -22,7 +22,7 @@ from statistics import mean
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
-    from .platooning_vehicle import PlatooningVehicle  # noqa 401
+    from plafosim.platooning_vehicle import PlatooningVehicle  # noqa 401
 
 LOG = logging.getLogger(__name__)
 
@@ -33,11 +33,14 @@ class Platoon:
     """
 
     def __init__(
-            self,
-            platoon_id: int,
-            formation: list,
-            desired_speed: float):
+        self,
+        platoon_id: int,
+        formation: list,
+        desired_speed: float,
+    ):
         """
+        Initialize a platoon instance.
+
         Parameters
         ----------
         platoon_id : int
@@ -224,6 +227,10 @@ class Platoon:
         ----------
         vehicle : PlatooningVehicle
             The considered vehicle within the platoon.
+
+        Returns
+        -------
+        int : The index of the member within the platoon
         """
 
         return self._formation.index(vehicle)
@@ -236,12 +243,15 @@ class Platoon:
         ----------
         vehicle : PlatooningVehicle
             The considered vehicle within the platoon.
+
+        Returns
+        -------
+        PlatooningVehicle : The member in the front
         """
 
         if vehicle is not self.leader:
             return self._formation[self.get_member_index(vehicle) - 1]
-        else:
-            return None
+        return None
 
     def get_back(self, vehicle: 'PlatooningVehicle'):
         """
@@ -251,12 +261,15 @@ class Platoon:
         ----------
         vehicle : PlatooningVehicle
             The considered vehicle within the platoon.
+
+        Returns
+        -------
+        PlatooningVehicle : The member in the back
         """
 
         if vehicle is not self.last:
             return self._formation[self.get_member_index(vehicle) + 1]
-        else:
-            return None
+        return None
 
     def update_desired_speed(self):
         """
@@ -270,11 +283,19 @@ class Platoon:
         LOG.debug(f"Updated platoon {self.platoon_id}'s desired speed to {self.desired_speed} (from {old_desired_speed})")
 
     def update_limits(self):
+        """
+        Update mobility limits for the platoon.
+        """
+
         self.update_max_speed()
         self.update_max_acceleration()
         self.update_max_deceleration()
 
     def update_cf_target_speed(self):
+        """
+        Update the cf target speed for the platoon.
+        """
+
         for member in self._formation:
             member._cf_target_speed = self._desired_speed
 
