@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2020-2022 Julian Heinovski <heinovski@ccs-labs.org>
+# Copyright (c) 2020-2025 Julian Heinovski <heinovski@ccs-labs.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
@@ -22,25 +22,26 @@ import math
 import sys
 from typing import TYPE_CHECKING
 
-from .algorithms.speed_position import SpeedPosition  # noqa 401
-from .gui import change_gui_vehicle_color
-from .message import Message, PlatoonAdvertisement
-from .mobility import CF_Model
-from .platoon import Platoon
-from .platoon_role import PlatoonRole
-from .statistics import (
+from plafosim.algorithms import *  # noqa 401
+from plafosim.gui import change_gui_vehicle_color
+from plafosim.mobility import CF_Model, is_gap_safe
+from plafosim.platoon import Platoon
+from plafosim.platoon_role import PlatoonRole
+from plafosim.statistics import (
     record_platoon_formation,
     record_platoon_trace,
     record_platoon_trip,
+    record_vehicle_change,
     record_vehicle_platoon_maneuvers,
     record_vehicle_platoon_trace,
+    record_vehicle_teleport,
 )
-from .util import round_to_next_base
-from .vehicle import Vehicle
-from .vehicle_type import VehicleType
+from plafosim.util import round_to_next_base
+from plafosim.vehicle import Vehicle
+from plafosim.vehicle_type import VehicleType
 
 if TYPE_CHECKING:
-    from .simulator import Simulator  # noqa 401
+    from plafosim.simulator import Simulator  # noqa 401
 
 LOG = logging.getLogger(__name__)
 
@@ -60,8 +61,8 @@ class PlatooningVehicle(Vehicle):
             desired_speed: float,
             depart_lane: int,
             depart_speed: float,
-            depart_time: int,
-            depart_delay: int = 0,
+            depart_time: float,
+            depart_delay: float = 0,
             communication_range: int = 1000,  # TODO use defaults
             acc_headway_time: float = 1.0,  # TODO use defaults
             cacc_spacing: float = 5.0,  # TODO use defaults
@@ -83,18 +84,18 @@ class PlatooningVehicle(Vehicle):
         vehicle_type : VehicleType
             The vehicle type of the vehicle
         depart_position : int
-            The depart position of the vehicle
+            The departure position of the vehicle
         arrival_position : int
             The arrival position of the vehicle
         desired_speed : float
             The desired driving speed of the vehicle
         depart_lane : int
-            The depart lane of the vehicle
+            The departure lane of the vehicle
         depart_speed : float
-            The depart speed of the vehicle
-        depart_time : int
-            The depart time of the vehicle
-        depart_delay : int
+            The departure speed of the vehicle
+        depart_time : float
+            The actual departure time of the vehicle
+        depart_delay : float
             The time the vehicle had to wait before starting its trip
         communication_range : int
             The maximum communication range of the vehicle
@@ -145,11 +146,12 @@ class PlatooningVehicle(Vehicle):
             try:
                 self._formation_algorithm = globals()[formation_algorithm](self, **kw_args)
             except KeyError:
-                sys.exit(f"ERROR: Unknown formation algorithm {formation_algorithm}! Did you import it?")
+                sys.exit(f"ERROR [{__name__}]: Unknown formation algorithm {formation_algorithm}! Did you import it?")
             self._execution_interval = execution_interval
 
             # initialize timers
             if pre_filled:
+                # FIXME float step lenghts not supported!
                 self._last_formation_step = simulator._rng.randint(0, self._execution_interval - 1)
             else:
                 self._last_formation_step = self._depart_time  # initialize with vehicle start
@@ -203,6 +205,8 @@ class PlatooningVehicle(Vehicle):
         # formation statistics
         self._formation_iterations = 0
         self._candidates_found = 0
+        self._candidates_found_individual = 0
+        self._candidates_found_platoon = 0
         self._candidates_filtered = 0
         self._candidates_filtered_follower = 0
         self._candidates_filtered_maneuver = 0
@@ -225,8 +229,8 @@ class PlatooningVehicle(Vehicle):
 
         if self._cf_model == CF_Model.ACC:
             return self._acc_headway_time
-        elif self._cf_model == CF_Model.CACC:
-            return -1
+        if self._cf_model == CF_Model.CACC:
+            return 0
         return super().desired_headway_time
 
     @property
@@ -334,7 +338,9 @@ class PlatooningVehicle(Vehicle):
 
     @property
     def color(self) -> tuple:
-        """Return the current color of the vehicle."""
+        """
+        Return the current color of the vehicle.
+        """
 
         return self._platoon.leader._color if self.is_in_platoon() else self._color
 
@@ -352,13 +358,17 @@ class PlatooningVehicle(Vehicle):
             The emission factors to use for current emission variable to be calculated
         scale : float
             The scale to normalize the calculated value
+
+        Returns
+        -------
+        float : The calculcated emission in ml/mg per s
         """
 
         # calculate impact of platooning on air drag based on
         # Charles-Henri Bruneau, Khodor Khadra and Iraj Mortazavi,
         # "Flow analysis of square-back simplified vehicles in platoon,"
         # International Journal of Heat and Fluid Flow, vol. 66, pp. 43–59,
-        # August 2017. Table 5: d = L, vehicle length = 4m, distance = 5m
+        # August 2017. Table 5: d = L, vehicle length = 5m, distance = 5m
         air_drag_change = 0.0
         if self._platoon_role == PlatoonRole.LEADER:
             # savings by followers
@@ -391,7 +401,7 @@ class PlatooningVehicle(Vehicle):
         This includes leaving the platoon and mostly statistics recording.
         """
 
-        if (self._position < self._arrival_position):
+        if self._position < self._arrival_position:
             LOG.warning(f"{self._vid}'s finish method was called even though vehicle did not arrive yet!")
             return
 
@@ -428,6 +438,16 @@ class PlatooningVehicle(Vehicle):
             if self._formation_iterations > 0
             else 0
         )
+        candidates_found_individual_avg = (
+            self._candidates_found_individual / self._formation_iterations
+            if self._formation_iterations > 0
+            else 0
+        )
+        candidates_found_platoon_avg = (
+            self._candidates_found_platoon / self._formation_iterations
+            if self._formation_iterations > 0
+            else 0
+        )
 
         assert platoon_time_ratio >= 0
         assert platoon_distance_ratio >= 0
@@ -464,16 +484,18 @@ class PlatooningVehicle(Vehicle):
                 basename=self._simulator._result_base_filename,
                 vehicle=self,
                 candidates_found_avg=candidates_found_avg,
+                candidates_found_individual_avg=candidates_found_individual_avg,
+                candidates_found_platoon_avg=candidates_found_platoon_avg,
                 candidates_filtered_avg=candidates_filtered_avg,
             )
 
-    def _action(self, step: int):
+    def _action(self, step: float):
         """
         Triggers specific actions of a PlatooningVehicle.
 
         Parameters
         ----------
-        step : int
+        step : float
             The current simulation step
         """
 
@@ -493,9 +515,6 @@ class PlatooningVehicle(Vehicle):
 
         # do platoon formation
         if self._formation_algorithm:
-            # transmit regular platoon advertisements
-            self._advertise()
-
             # execute formation algorithm at every execution interval
             if step >= self._last_formation_step + self._execution_interval:
                 # search for a platoon (depending on the algorithm)
@@ -532,26 +551,33 @@ class PlatooningVehicle(Vehicle):
 
         if self._simulator._record_platoon_traces:
             # write statistics about the current platoon
-            record_vehicle_platoon_trace(
-                basename=self._simulator._result_base_filename,
-                step=self._simulator.step,
-                vehicle=self
-            )
-
-            if self._platoon_role is PlatoonRole.LEADER:
-                # write statistics about the platoon
+            if self._platoon.leader is self:
+                # we do not want followers to record that
+                assert self._platoon_role != PlatoonRole.FOLLOWER
                 record_platoon_trace(
                     basename=self._simulator._result_base_filename,
                     step=self._simulator.step,
-                    vehicle=self
+                    vehicle=self,
                 )
 
+        if self._simulator._record_vehicle_platoon_traces:
+            # write statistics about this vehicle within the current platoon
+            record_vehicle_platoon_trace(
+                basename=self._simulator._result_base_filename,
+                step=self._simulator.step,
+                vehicle=self,
+            )
+
     # TODO rework to only include "neighbors" and move platoon extraction to formation algorithm
-    def _get_available_platoons(self):
+    def _get_available_platoons(self) -> list:
         """
         Return the available platoon candidates of the vehicle.
 
         This imitates neighbor maintenance by using a neighbor table.
+
+        Returns
+        -------
+        list(PlatooninVehicle) : The list of available platoons
         """
 
         # HACK FOR AVOIDING MAINTAINING A NEIGHBOR TABLE (for now)
@@ -566,15 +592,30 @@ class PlatooningVehicle(Vehicle):
             if not isinstance(vehicle, PlatooningVehicle):
                 continue
 
-            # filter non-platoons
-            if vehicle.platoon_role != PlatoonRole.LEADER and vehicle.platoon_role != PlatoonRole.NONE:
-                continue
-
             # filter non-available vehicles based on communication range
             if abs(vehicle.position - self._position) > self._communication_range:
                 LOG.trace(f"{self._vid}'s neighbor {vehicle.vid} is out of communication range ({self._communication_range}m)")
                 continue
 
+            # filter non-platoons
+            # this mimics advertisements that are only done by platoon leaders or individual vehicles
+            # this does not include outdated information, e.g., due to an individual vehicle not available anymore
+            # disabling this increases the number of false positives, thereby increasing the number of failed join maneuvers
+            if self._simulator._distributed_platoon_knowledge:
+                if vehicle.platoon_role != PlatoonRole.LEADER and vehicle.platoon_role != PlatoonRole.NONE:
+                    self._candidates_filtered += 1
+                    self._candidates_filtered_follower += 1
+                    continue
+
+            # filter vehicles which are already in a maneuver
+            # disabling this increases the number of false positives, thereby increasing the number of failed join maneuvers
+            if self._simulator._distributed_maneuver_knowledge:
+                if vehicle.in_maneuver:
+                    LOG.trace(f"{vehicle.vid} is already in a maneuver")
+                    self._candidates_filtered += 1
+                    self._candidates_filtered_maneuver += 1
+                    continue
+
             platoons.append(vehicle.platoon)
 
         return platoons
@@ -631,7 +672,7 @@ class PlatooningVehicle(Vehicle):
             self._joins_aborted += 1
             self._joins_aborted_front += 1
             return
-        elif self._position > leader.platoon.last.position:
+        if self._position > leader.platoon.last.position:
             # TODO join at (arbitrary position) in the middle
             LOG.debug(f"{self._vid} is in front of (at least) the last vehicle {leader.platoon.last.vid} of the target platoon {platoon_id} ({leader_id})")
             LOG.warning("Join at arbitrary positions of a platoon is not yet implemented! Aborting the join maneuver!")
@@ -661,7 +702,7 @@ class PlatooningVehicle(Vehicle):
             return
 
         if new_position < self._depart_position:
-            LOG.warning(f"{self._vid}'s new position would be before its depart position! Aborting the join maneuver!")
+            LOG.warning(f"{self._vid}'s new position would be before its departure position! Aborting the join maneuver!")
             self.in_maneuver = False
             self._platoon_role = PlatoonRole.NONE
 
@@ -690,7 +731,7 @@ class PlatooningVehicle(Vehicle):
             return
 
         if self.max_speed <= leader.platoon.speed:
-            # we will never be able to approach the platoon
+            # we will never be able to approach the platoon or maintain the platoon speed
             LOG.warning(f"{self._vid}'s maximum speed is too low such that it can never reach the platoon! Aborting the join maneuver!")
             self.in_maneuver = False
             self._platoon_role = PlatoonRole.NONE
@@ -738,6 +779,8 @@ class PlatooningVehicle(Vehicle):
             total_approach_time = round_to_next_base(total_approach_time, self._simulator.step_length)
             # the platoon will be driving while we are waiting
             new_position += total_approach_time * leader.platoon.speed
+            # FIXME the extra distance from above increases the distance to approach, which should also increase the total approaching time (by time to drive this extra distance with maximum speed)
+            # TODO use classical formulas from school physics: when do 2 cars meet and where?
             if new_position >= self._simulator.road_length:
                 LOG.warning(f"{self._vid}'s new position would be outside of the road! Aborting the join maneuver")
                 self.in_maneuver = False
@@ -773,6 +816,8 @@ class PlatooningVehicle(Vehicle):
 
     def _join_teleport(self, leader: 'PlatooningVehicle', last: 'PlatooningVehicle', new_position: float):
         """
+        Perform the actual teleporting of the join maneuver.
+
         Parameters
         ----------
         leader : PlatooningVehicle
@@ -800,7 +845,10 @@ class PlatooningVehicle(Vehicle):
         assert self._in_maneuver
         assert self._platoon_role is PlatoonRole.JOINER
         assert leader.in_maneuver
-        assert (leader.platoon_role is PlatoonRole.LEADER or leader.platoon_role is PlatoonRole.NONE)
+        assert (
+            leader.platoon_role is PlatoonRole.LEADER
+            or leader.platoon_role is PlatoonRole.NONE
+        )
 
         # re-calculate new position
         new_position = last.rear_position - self._cacc_spacing
@@ -820,12 +868,12 @@ class PlatooningVehicle(Vehicle):
             # MAKE SPACE FOR THE JOINER
             # the idea is to move the vehicle(s) behind the platoon to make room for the joiner
             # we start with the first one (direct follower) and proceed with the next vehicle(s) as long as we need more space
-            # thus, we move only as little vehicles and as little as necessary (until they reach the mingap to the front vehicle)
+            # thus, we move only as little vehicles and as little as necessary (until they reach the minimum gap to the front vehicle)
             # this way, it is more realistic and keeps the correct order of vehicles without producing an incorrect state during the process
             # we might also move the method to the vehicle class or make it even part of the simulator
 
             # how big needs the gap behind the platoon in front of the platoon successor to be
-            # cacc spacing + joiner + successor's min gap
+            # CACC spacing + joiner + successor's min gap
             required_gap = self._cacc_spacing + self.length + platoon_successor.min_gap
             LOG.trace(f"We need a gap of {required_gap}m behind the platoon (vehicle {last.vid}) to teleport vehicle {self._vid}")
             current_gap = last.rear_position - platoon_successor.position
@@ -932,6 +980,7 @@ class PlatooningVehicle(Vehicle):
             leader._platoon_role = PlatoonRole.LEADER
         else:
             assert leader.platoon_role is PlatoonRole.LEADER
+        leader.platoon._formation.append(self)
 
         # update self
         self._platoon_role = PlatoonRole.FOLLOWER
@@ -957,7 +1006,6 @@ class PlatooningVehicle(Vehicle):
         self._number_platoons += 1
 
         # update all members
-        leader.platoon._formation.append(self)
         # update the desired speed of the platoon to the average of all platoon members
         if self._simulator._update_desired_speed:
             leader.platoon.update_desired_speed()
@@ -968,6 +1016,7 @@ class PlatooningVehicle(Vehicle):
             # thus, the follower now drives as fast as the already existing platoon
             # (i.e., only the leader in the worst case)
             vehicle._platoon = leader.platoon
+            # TODO should be replaced by call to platoon.update_cf_target_speed()
             vehicle._cf_target_speed = vehicle._platoon.desired_speed
 
         LOG.debug(f"{self._vid} joined platoon {leader.platoon.platoon_id} (leader: {leader.vid})")
@@ -981,11 +1030,14 @@ class PlatooningVehicle(Vehicle):
 
         Parameters
         ----------
-
         target_position : float
             The target position to approach
         target_speed : float
             The target speed after approaching
+
+        Returns
+        -------
+        float : The calculated approaching time
         """
 
         initial_distance = target_position - self._position
@@ -1012,6 +1064,7 @@ class PlatooningVehicle(Vehicle):
             else:
                 # we need to decelerate to approach the platoon
                 # the time we need to decelerate to target speed (given a linear deceleration)
+                # TODO do not use maximum deceleration for maneuver
                 time_deceleration = (self._speed - target_speed) / self.max_deceleration
                 # the distance driven while decelerating to the target speed (given a linear acceleration
                 distance_deceleration = (self._speed + target_speed) / 2 * time_deceleration
@@ -1023,7 +1076,7 @@ class PlatooningVehicle(Vehicle):
                 total_approach_time = time_current_speed + time_deceleration
         else:
             # we do not need to consider this case as our error is only between 0m and last.length + cacc_spacing (e.g., 9m)
-            assert abs(initial_distance) <= self.length + self._cacc_spacing  # FIXME use length of last vehicle in platoon
+            assert abs(initial_distance) <= self.length + self._cacc_spacing  # TODO use length of last vehicle in platoon
             total_approach_time = 0  # FIXME: we ignore that for now, since the time should be very small anyhow
 
         return total_approach_time
@@ -1038,7 +1091,7 @@ class PlatooningVehicle(Vehicle):
             The new position
         new_lane : int
             The new lane
-        new_speed : int
+        new_speed : float
             The new speed
         """
 
@@ -1059,6 +1112,19 @@ class PlatooningVehicle(Vehicle):
             LOG.trace(f"{self._vid} changed speed to {self._speed}m/s (from {current_speed}m/s)")
             self._joins_teleport_speed += 1
 
+        if self._simulator._record_vehicle_teleports:
+            record_vehicle_teleport(
+                basename=self._simulator._result_base_filename,
+                step=self._simulator.step,
+                vid=self._vid,
+                old_position=current_position,
+                old_lane=current_lane,
+                old_speed=current_speed,
+                new_position=new_position,
+                new_lane=new_lane,
+                new_speed=new_speed,
+            )
+
     def _leave(self):
         """
         Lets a vehicle leave a platoon.
@@ -1130,6 +1196,14 @@ class PlatooningVehicle(Vehicle):
                 new_leader = self._platoon.formation[1]
                 new_leader._platoon_role = PlatoonRole.LEADER
                 new_leader._cf_model = CF_Model.ACC
+                # platoon is changed to current leader_id
+                new_leader._platoon._platoon_id = new_leader.vid
+
+                # reset color of all remaining vehicle
+                if self._simulator._gui and self._simulator.step >= self._simulator._gui_start:
+                    for vehicle in self.platoon.formation[1:]:
+                        change_gui_vehicle_color(vehicle.vid, new_leader._color)
+
                 LOG.debug(f"{new_leader.vid} became leader of platoon {new_leader.platoon.platoon_id}")
         elif self is self._platoon.last:
             # leave at back
@@ -1168,12 +1242,39 @@ class PlatooningVehicle(Vehicle):
                 # TODO this looks strange in the GUI, since we do this before actually leaving the formation
                 self._platoon_role = PlatoonRole.LEAVER
                 self._cf_model = CF_Model.ACC
-                # FIXME Missing method. TODO Implement with new function!
-                sys.exit(f"ERROR: This is not implemented (anymore)! {__file__}:{sys._getframe().f_lineno}")
-                if not self._simulator._change_lane(self, self._lane + 1, "maneuver"):
-                    # could not leave the platoon
+
+                if not self._left_lane_blocked():
+                    # leave current platoon by changing to next lane
+                    self._lane += 1
+
+                    # write the lane change into related statistics
+                    if self._simulator._record_vehicle_changes:
+                        record_vehicle_change(
+                            basename=self._simulator._result_base_filename,
+                            step=self._simulator.step,
+                            vid=self.vid,
+                            position=self.position,
+                            speed=self.speed,
+                            source_lane=self.lane - 1,
+                            target_lane=self.lane,
+                            reason="maneuver",
+                        )
+                    LOG.trace(f"{self.vid} left platoon {self.platoon.platoon_id} by changing to lane {self.lane}.")
+
+                else:
+                    # the left lane is not available for leaving
+                    # revert leaving process
+                    self.in_maneuver = False
+                    leader.in_maneuver = False
+                    self._leaves_arbitrary -= 1
+                    self._leaves_aborted += 1
+                    self._platoon_role = PlatoonRole.FOLLOWER
+                    self._cf_model = CF_Model.CACC
+
                     # TODO this could be just a return in future to let the leaver try again
-                    sys.exit(f"ERROR: Could not move vehicle {self._vid} to the adjacent lane to leave the platoon {self._platoon.platoon_id}!")
+                    #sys.exit(f"ERROR [{__name__}]: Could not move vehicle {self._vid} to the adjacent lane to leave platoon {self.platoon.platoon_id}!")
+                    LOG.trace(f"Could not move vehicle {self._vid} to the adjacent lane to leave platoon {self.platoon.platoon_id}!")
+                    return
 
             # move all remaining platoon members further to the front
             front = self._platoon.get_front(self)
@@ -1190,7 +1291,7 @@ class PlatooningVehicle(Vehicle):
                     continue
                 LOG.trace(f"Moving follower {vehicle.vid} from {vehicle.position} by {gap_error}m ")
                 vehicle._position += gap_error
-                LOG.trace(f"{vehicle.vid} is not at {vehicle.position}")
+                LOG.trace(f"{vehicle.vid} is now at {vehicle.position}")
                 follower_gap = front.rear_position - vehicle.position
                 # avoid issues due to floating point precision
                 assert math.isclose(follower_gap, vehicle._cacc_spacing)
@@ -1202,6 +1303,7 @@ class PlatooningVehicle(Vehicle):
         if self._simulator._update_desired_speed:
             self._platoon.update_desired_speed()
         self._platoon.update_limits()
+        self._platoon.update_cf_target_speed()
 
         # update formation for all (remaining) members
         # TODO this could be nicer, e.g., by taking the (new) leader's updated formation
@@ -1214,6 +1316,7 @@ class PlatooningVehicle(Vehicle):
         LOG.debug(f"{self._vid} left platoon {self._platoon.platoon_id} (leader {self._platoon.leader.vid})")
         self._platoon_role = PlatoonRole.NONE  # the current platoon role
         self._platoon = Platoon(self._vid, [self], self._desired_speed)  # use explicit individual desired speed
+        self._cf_target_speed = self._desired_speed  # we reset the cf_target_speed
         self._cf_model = CF_Model.ACC  # not necessary, but we still do it explicitly
 
         # reset color of vehicle
@@ -1231,79 +1334,61 @@ class PlatooningVehicle(Vehicle):
         self._distance_in_platoon += self._position - self._last_platoon_join_position
         self._leaves_successful += 1
 
-    def _advertise(self):
+    def _left_lane_blocked(self) -> bool:
         """
-        Maintains regular sending of platoon advertisements.
+        Check whether a vehicle can move to the left lane in order to leave its current platoon.
 
-        Advertisement are in general not used at the moment.
-
-        THIS IS DEPRECATED AT THE MOMENT!!!
+        Returns
+        -------
+        bool : Whether the left lane is blocked
         """
+        # TODO prime example for use of pandas Dataframe
 
-        return  # TODO this is not necessary as a perfect communication guarantees information
+        # make sure lane is within road bounds
+        if self._simulator.number_of_lanes <= self._lane + 1:
+            LOG.trace(
+                f"ERROR [{__name__}]: Could not move vehicle {self.vid} to the adjacent lane to leave the platoon "
+                f"{self.platoon.platoon_id} because the platoon is already driving on the leftmost lane!"
+            )
+            return True
 
-        advertisement_interval = 600  # in s # TODO make parameter
-        if (
-            not self._last_advertisement_step
-            or self._last_advertisement_step + advertisement_interval <= self._simulator.step
-        ):
-            self._send_advertisements()
-            self._last_advertisement_step = self._simulator.step
+        # TODO same logic as in spawning and lane changing -> avoid duplicated code
 
-    def _send_advertisements(self):
-        """
-        Transmit a broadcast to advertise as platoon.
+        # find closest vehicle on left lane (forwards)
+        closest_front = min(
+            [vehicle for vehicle in self._simulator._vehicles.values() if
+             vehicle.position >= self.position and vehicle.lane == (self.lane + 1)],
+            key=lambda vehicle: vehicle.position,
+            default=None,
+        )
+        # find closest vehicle on left lane (backwards)
+        closest_back = max(
+            [vehicle for vehicle in self._simulator._vehicles.values() if
+             vehicle.position < self.position and vehicle.lane == (self.lane + 1)],
+            key=lambda vehicle: vehicle.position,
+            default=None,
+        )
+        front_gap_safe = not closest_front or is_gap_safe(
+            front_position=closest_front._position,
+            front_speed=closest_front.speed,
+            front_max_deceleration=closest_front.max_deceleration,
+            front_length=closest_front.length,
+            back_position=self.position,
+            back_speed=self.speed,
+            back_max_acceleration=self.max_acceleration,
+            back_min_gap=self.min_gap,
+            step_length=self._simulator.step_length,
+        )
+        back_gap_safe = not closest_back or is_gap_safe(
+            front_position=self.position,
+            front_speed=self.speed,
+            front_max_deceleration=self.max_deceleration,
+            front_length=self.length,
+            back_position=closest_back.position,
+            back_speed=closest_back.speed,
+            back_max_acceleration=closest_back.max_acceleration,
+            back_min_gap=closest_back.min_gap,
+            step_length=self._simulator.step_length,
+        )
 
-        Advertisement are in general not used at the moment.
-
-        THIS IS DEPRECATED AT THE MOMENT!!!
-        """
-
-        for vehicle in self._simulator._vehicles.values():
-            self._transmit(-1, PlatoonAdvertisement(
-                self._vid,
-                vehicle.vid,
-                self._vid,
-                self._vid,
-                self._speed,
-                self._lane,
-                self._vid,
-                self._position,
-                self.rear_position,
-            ))
-
-    def _handle_message(self, message: Message):
-        """
-        Handle a message of arbitrary type Message.
-
-        Messages are in general not used at the moment.
-
-        Parameters
-        ----------
-        message : Message
-            The message to be handled
-
-        THIS IS DEPRECATED AT THE MOMENT!!!
-        """
-
-        func = self.__class__.__dict__.get(
-            '_receive_' + message.__class__.__name__,
-            super().__dict__.get('_handle_message'))
-        return func(self, message)
-
-    def _receive_PlatoonAdvertisement(self, advertisement: PlatoonAdvertisement):
-        """
-        Handle a message of the specific type PlatoonAdvertisement.
-
-        Messages are in general not used at the moment.
-
-        Parameters
-        ----------
-        advertisement : PlatoonAdvertisement
-            The advertisement to be received
-
-        THIS IS DEPRECATED AT THE MOMENT!!!
-        """
-
-        # TODO add contents to the neighbor table
-        LOG.debug(f"{self._vid} received an advertisement from {advertisement.origin}")
+        return not front_gap_safe & back_gap_safe
