@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2020-2023 Julian Heinovski <heinovski@ccs-labs.org>
+# Copyright (c) 2020-2025 Julian Heinovski <heinovski@ccs-labs.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
@@ -22,19 +22,17 @@ import time
 from typing import TYPE_CHECKING
 
 from plafosim import __version__
-
-from .formation_algorithm import FormationAlgorithm
-from .util import rgb2hex
+from plafosim.util import rgb2hex
 
 if TYPE_CHECKING:
-    from .platooning_vehicle import PlatooningVehicle  # noqa 401
-    from .simulator import Simulator  # noqa 401
-    from .vehicle import Vehicle  # noqa 401
+    from plafosim.platooning_vehicle import PlatooningVehicle  # noqa 401
+    from plafosim.simulator import Simulator  # noqa 401
+    from plafosim.vehicle import Vehicle  # noqa 401
 
 
 def record_general_data_begin(basename: str, simulator: 'Simulator'):
     assert basename
-    from .simulator import Simulator
+    from plafosim.simulator import Simulator
     assert isinstance(simulator, Simulator)
     with open(f'{basename}_general.out', 'w') as f:
         f.write(f"simulation start: {time.asctime(time.localtime(time.time()))}\n")
@@ -45,7 +43,7 @@ def record_general_data_begin(basename: str, simulator: 'Simulator'):
 
 def record_general_data_end(basename: str, simulator: 'Simulator'):
     assert basename
-    from .simulator import Simulator
+    from plafosim.simulator import Simulator
     assert isinstance(simulator, Simulator)
     with open(f'{basename}_general.out', 'a') as f:
         f.write(f"simulation end: {time.asctime(time.localtime(time.time()))}\n")
@@ -286,6 +284,10 @@ def initialize_platoon_formation(basename: str):
             "formationIterations,"
             "candidatesFound,"
             "candidatesFoundAvg,"
+            "candidatesFoundIndividual,"
+            "candidatesFoundPlatoon,"
+            "candidatesFoundIndividualAvg,"
+            "candidatesFoundPlatoonAvg,"
             "candidatesFiltered,"
             "candidatesFilteredAvg,"
             "candidatesFilteredFollower,"
@@ -298,6 +300,8 @@ def record_platoon_formation(
     basename: str,
     vehicle: 'PlatooningVehicle',
     candidates_found_avg: float,
+    candidates_found_individual_avg: float,
+    candidates_found_platoon_avg: float,
     candidates_filtered_avg: float
 ):
     assert basename
@@ -310,6 +314,10 @@ def record_platoon_formation(
             f"{vehicle._formation_iterations},"
             f"{vehicle._candidates_found},"
             f"{candidates_found_avg},"
+            f"{vehicle._candidates_found_individual},"
+            f"{vehicle._candidates_found_platoon},"
+            f"{candidates_found_individual_avg},"
+            f"{candidates_found_platoon_avg},"
             f"{vehicle._candidates_filtered},"
             f"{candidates_filtered_avg},"
             f"{vehicle._candidates_filtered_follower},"
@@ -318,42 +326,6 @@ def record_platoon_formation(
         )
 
 
-def initialize_infrastructure_assignments(basename: str):
-    assert basename
-    with open(f'{basename}_infrastructure_assignments.csv', 'w') as f:
-        f.write(
-            "id,"
-            "assignmentsSolved,"
-            "assignmentsNotSolvable,"
-            "assignmentsSolvedOptimal,"
-            "assignmentsSolvedFeasible,"
-            "assignmentsNone,"
-            "assignmentsSelf,"
-            "assignmentsCandidateJoinedAlready,"
-            "assignmentsVehicleBecameLeader,"
-            "assignmentsSuccessful"
-            "\n"
-        )
-
-
-def record_infrastructure_assignments(basename: str, iid: int, algorithm: FormationAlgorithm):
-    assert basename
-    assert isinstance(algorithm, FormationAlgorithm)
-    with open(f'{basename}_infrastructure_assignments.csv', 'a') as f:
-        f.write(
-            f"{iid},"
-            f"{algorithm._assignments_solved},"
-            f"{algorithm._assignments_not_solvable},"
-            f"{algorithm._assignments_solved_optimal},"
-            f"{algorithm._assignments_solved_feasible},"
-            f"{algorithm._assignments_none},"
-            f"{algorithm._assignments_self},"
-            f"{algorithm._assignments_candidate_joined_already},"
-            f"{algorithm._assingments_vehicle_became_leader},"
-            f"{algorithm._assignments_successful}"
-            "\n"
-        )
-
 # traces
 
 
@@ -572,9 +544,9 @@ def record_platoon_change(
     reason: str,
 ):
     assert basename
-    from .platooning_vehicle import PlatooningVehicle
+    from plafosim.platooning_vehicle import PlatooningVehicle
     assert isinstance(leader, PlatooningVehicle)
-    from .platoon_role import PlatoonRole
+    from plafosim.platoon_role import PlatoonRole
     assert leader.platoon_role == PlatoonRole.LEADER and leader.platoon.leader.vid == leader.vid
     with open(f'{basename}_platoon_changes.csv', 'a') as f:
         f.write(
