@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2020-2023 Julian Heinovski <heinovski@ccs-labs.org>
+# Copyright (c) 2020-2025 Julian Heinovski <heinovski@ccs-labs.org>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
 #
@@ -20,8 +20,8 @@
 import logging
 from typing import TYPE_CHECKING
 
-from .mobility import CF_Model
-from .statistics import (
+from plafosim.mobility import CF_Model
+from plafosim.statistics import (
     record_emission_trace_prefix,
     record_emission_trace_suffix,
     record_emission_trace_value,
@@ -29,8 +29,8 @@ from .statistics import (
     record_vehicle_trace,
     record_vehicle_trip,
 )
-from .util import speed2distance
-from .vehicle_type import VehicleType
+from plafosim.util import speed2distance
+from plafosim.vehicle_type import VehicleType
 
 if TYPE_CHECKING:
     from numpy.typing import ArrayLike
@@ -120,7 +120,7 @@ class Vehicle:
         self._speed = self._depart_speed  # the current speed of the vehicle
         self._blocked_front = False  # whether the vehicle is blocked by a slower vehicle in front
         self._acceleration = 0  # the current acceleration of the vehicle, used (only) for the emission model
-        self._cf_model = CF_Model.Human  # the current car following model
+        self._cf_model = CF_Model.HUMAN  # the current car following model
         self._cf_target_speed = desired_speed  # the target speed for the car following
 
         # communication properties
@@ -478,10 +478,10 @@ class Vehicle:
                     scale *= 742.0
             value = (
                 self._calculate_emission(
-                    self._acceleration,
-                    self._speed,
-                    ec.emission_factors[variable],
-                    scale,
+                    a=self._acceleration,
+                    v=self._speed,
+                    f=ec.emission_factors[variable],
+                    scale=scale,
                 )
                 * self._simulator.step_length
             )
@@ -507,6 +507,10 @@ class Vehicle:
             The emission factors to use for current emission variable to be calculated
         scale : float
             The scale to normalize the calculated value
+
+        Returns
+        -------
+        float : The calculcated emission in ml/mg per s
         """
 
         if a < 0:
